package cn.kdan.pdf.tech.core.controller;

import cn.kdan.pdf.tech.core.aspect.PreCheckRole;
import cn.kdan.pdf.tech.core.constant.VppTeamConstant;
import cn.kdan.pdf.tech.core.enums.RoleEnum;
import cn.kdan.pdf.tech.core.model.VppTeam;
import cn.kdan.pdf.tech.core.pojo.dto.AddTeamMemberDTO;
import cn.kdan.pdf.tech.core.pojo.dto.CreateTeamDTO;
import cn.kdan.pdf.tech.core.pojo.dto.EditTeamDTO;
import cn.kdan.pdf.tech.core.pojo.vo.TeamInfoVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamListVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamVO;
import cn.kdan.pdf.tech.core.service.VppRTeamMemberRoleService;
import cn.kdan.pdf.tech.core.service.VppTeamService;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.util.MapUtils;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageInfo;
import constant.CommonConstant;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pojo.ResultMap;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;


/**
 * @author tangxiangan
 */
@RestController
@RequestMapping("/vppTeam")
@Validated
public class  VppTeamController {

    @Resource
    private VppTeamService vppTeamService;
    @Resource
    private VppRTeamMemberRoleService vppRTeamMemberRoleService;


    /**
     * 团队管理列表
     * @param teamId 团队id
     * @param keyword 模糊查询关键字（团队描述、团队管理员）
     * @return 团队数据
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @GetMapping("/getManageTeamList")
    public ResultMap<PageInfo<VppTeamVO>> getManageTeamList(
            @RequestParam(required = false, defaultValue = "1") Integer page,
            @RequestParam(required = false, defaultValue = "10") Integer pageSize,
            @RequestParam(required = false) String teamId,
            @RequestParam(required = false) String keyword) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppTeamService.getManageTeamList(page, pageSize, teamId, keyword));
    }

    /**
     * 团队管理列表导出
     * @param teamId 团队id
     * @param keyword 模糊查询关键字
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @GetMapping("/exportManageTeamList")
    public void exportManageTeamList(@RequestParam(required = false) String teamId, @RequestParam(required = false) String keyword, HttpServletResponse response) throws IOException {
        try {
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
            String fileName = URLEncoder.encode("团队管理列表", "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            // 这里需要设置不关闭流
            EasyExcel.write(response.getOutputStream(), VppTeamVO.class)
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    .autoCloseStream(Boolean.FALSE)
                    .sheet("团队管理列表")
                    .doWrite(() -> {
                        // 分页查询数据
                        return vppTeamService.exportManageTeamList(teamId, keyword);
                    });
        } catch (Exception e) {
            // 重置response
            response.reset();
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            Map<String, String> map = MapUtils.newHashMap();
            map.put("status", "failure");
            map.put("message", "下载文件失败" + e.getMessage());
            response.getWriter().println(JSON.toJSONString(map));
        }
    }

    /**
     * 创建团队
     * @param dto
     * @return
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @PostMapping("/createTeam")
    public ResultMap<Boolean> createTeam(@RequestBody CreateTeamDTO dto) {
        vppTeamService.createTeam(dto);
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_SEND_ADD_ADMIN_EMAIL, Boolean.TRUE);
    }

    /**
     * 编辑团队
     * @param dto
     * @return
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @PutMapping("/editTeam")
    public ResultMap<Boolean> editTeam(@RequestBody EditTeamDTO dto) {
        vppTeamService.editTeam(dto);
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_CHANGE_TEAM_MEMBER, Boolean.TRUE);
    }

    /**
     * 编辑团队时获取团队信息
     * @param teamId 团队id
     * @return
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @GetMapping("/getTeamProductInfo")
    public ResultMap<TeamInfoVO> getTeamProductInfo(@RequestParam String teamId) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppTeamService.getTeamProductInfo(teamId));
    }

    /**
     * 删除团队
     * @param teamId 团队id
     * @return
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @DeleteMapping("/deleteTeam")
    public ResultMap<Boolean> deleteTeam(@RequestParam String teamId) {
        vppTeamService.deleteTeam(teamId);
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_SEND_ADD_ADMIN_EMAIL, Boolean.TRUE);
    }

    /**
     * 添加团队管理员（如果用户还没有注册会先要求注册）
     *
     * @param teamIds 团队id
     * @param account 用户邮箱
     * @return Boolean
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @PostMapping("/sendEmailForAddTeamAdmin")
    public ResultMap<Boolean> sendEmailForAddTeamAdmin(@RequestParam @Size(min = 1,message = "teamIds cannot be blank") List<String> teamIds,
                                                       @RequestParam @NotBlank(message = "account cannot be blank") String account) {
        vppTeamService.sendEmailForAddTeamAdmin(teamIds,account);
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_SEND_ADD_ADMIN_EMAIL,Boolean.TRUE);
    }

    /**
     * 管理员管理中修改管理员管理的团队
     *
     * @param memberId 用户id
     * @param teamIds 团队id
     * @return boolean
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @PostMapping("/editTeamAdmin")
    public ResultMap<Boolean> editTeamAdmin( @RequestParam String memberId,@RequestParam List<String> teamIds) {
        vppTeamService.editTeamAdmin(memberId,teamIds);
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_CHANGE_TEAM_MEMBER,Boolean.TRUE);
    }

    /**
     * 用户点击邀请邮件之后调用 将用户加入团队
     *
     * @param memberId 用户id
     * @param teamIds 团队id
     * @param action 动作
     * @param code 验证码（邀请链接附带）
     * @return boolean
     */
    @PostMapping("/enterTeam")
    public ResultMap<Boolean> enterTeam( @RequestParam String memberId,@RequestParam List<String> teamIds,
                                             @RequestParam String action, @RequestParam String code) {
        vppTeamService.enterTeam(memberId,teamIds,action,code);
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_SEND_ADD_ADMIN_EMAIL,Boolean.TRUE);
    }

    /**
     * 查出用户管理的团队和公司下的所有团队（用于修改管理员时选择团队）
     *
     * @param memberId 用户id
     * @return VppTeamListVO
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @GetMapping("/listForAdminEdit")
    public ResultMap<VppTeamListVO> listForAdminEdit(@RequestParam @NotBlank(message = "memberId cannot be blank") String memberId) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppTeamService.listForAdminEdit(memberId));
    }

    /**
     * 查出用户管理的团队和公司下的所有团队（用于团队成员编辑选择团队）（用于添加管理员时选择团队）（用于序列码授权时选择团队）
     *
     * @return VppTeamListVO
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @GetMapping("/listWithAdmin")
    public ResultMap<List<VppTeam>> listWithAdmin() {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppTeamService.listWithAdmin());
    }

    /**
     * 添加团队成员
     * @return ResultMap
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @PostMapping("/addTeamMember")
    public ResultMap<Boolean> addTeamMember(@Valid @RequestBody AddTeamMemberDTO dto) {
        vppRTeamMemberRoleService.addTeamMember(dto);
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_SEND_ADD_ADMIN_EMAIL, Boolean.TRUE);
    }

    /**
     * 导入批量添加团队成员
     * @param file 文件
     * @return
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @PostMapping("/addTeamMemberBatch")
    public ResultMap<String> addTeamMemberBatch(@RequestParam("file") MultipartFile file) {
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_SEND_ADD_ADMIN_EMAIL, vppRTeamMemberRoleService.addTeamMemberBatch(file));
    }


    /**
     * 用户确认加入团队
     * @param memberEmail 用户邮件
     * @param memberName 用户名
     * @param teamId 团队id
     * @param action 动作
     * @param code 验证码
     * @return
     */
    @PostMapping("/confirmTeamMember")
    public ResultMap<String> confirmTeamMember(@RequestParam String memberEmail, @RequestParam String memberName, @RequestParam String teamId,
                                                @RequestParam String action, @RequestParam String code) {
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_SEND_ADD_ADMIN_EMAIL,
                vppRTeamMemberRoleService.confirmTeamMember(memberEmail, memberName, teamId, action, code));
    }

    /**
     * 编辑团队成员
     *
     * @param memberId 用户id
     * @param teamId 团队id
     * @param name 用户名
     * @return ResultMap
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @PostMapping("/editTeamMember")
    public ResultMap<Boolean> editTeamMember(@RequestParam String memberId,@RequestParam String teamId,@RequestParam String name) {
        vppRTeamMemberRoleService.editTeamMember(memberId,teamId,name);
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_CHANGE_TEAM_MEMBER, Boolean.TRUE);
    }

    /**
     * 删除团队普通用户或者管理员
     *
     * @param memberId 用户id
     * @return ResultMap
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @PostMapping("/deleteMember")
    public ResultMap<Boolean> deleteMember(@RequestParam String memberId) {
        vppRTeamMemberRoleService.deleteMember(memberId);
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_DELETE_TEAM_MEMBER, Boolean.TRUE);
    }

    @PreCheckRole(roles = {RoleEnum.SUPER_ADMIN})
    @PostMapping("/deleteAdmin")
    public ResultMap<Boolean> deleteAdmin(@RequestParam String memberId) {
        vppRTeamMemberRoleService.deleteAdmin(memberId);
        return new ResultMap<>(CommonConstant.SUCCESS, VppTeamConstant.SUCCESS_MSG_DELETE_TEAM_MEMBER, Boolean.TRUE);
    }

}
